﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Reflection;

/// <summary>
/// DBHelper 的摘要说明
/// </summary>
public class DBHelper
{
    #region class成员
    //连接对象
    public static readonly string conStr = "server=.;database=Test1;uid=sa;pwd=sa";//System.Configuration.ConfigurationManager.ConnectionStrings["Test"].ConnectionString;
    /// <summary>
    /// 返回读取器
    /// </summary>
    /// <param name="connectionString"></param>
    /// <param name="commandType"></param>
    /// <param name="commandText"></param>
    /// <param name="commandParameters"></param>
    /// <returns></returns>
    public static SqlDataReader ExecuteReader(CommandType commandType, string commandText, params SqlParameter[] commandParameters)
    {

        SqlConnection conn = new SqlConnection(conStr);
        try
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, commandType, conn, commandText, commandParameters);
            SqlDataReader sdr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
            cmd.Parameters.Clear();
            return sdr;
        }
        catch
        {
            throw new Exception("数据库发生异常");
        }
    }
    /// <summary>
    /// 查询单行记录
    /// </summary>
    /// <param name="commandType"></param>
    /// <param name="commandText"></param>
    /// <param name="commandParameters"></param>
    /// <returns></returns>
    public static object ExecuteScalar(CommandType commandType, string commandText, params SqlParameter[] commandParameters)
    {

        SqlConnection conn = new SqlConnection(conStr);
        SqlCommand cmd = null;
        try
        {
            cmd = new SqlCommand();
            PrepareCommand(cmd, commandType, conn, commandText, commandParameters);
            object val = cmd.ExecuteScalar();
            cmd.Parameters.Clear();
            return val;
        }
        catch (Exception er)
        {
            throw new Exception(er.Message);
        }
    }
    /// <summary>
    /// 增删改 返回受影响的行数
    /// </summary>
    /// <param name="commandType"></param>
    /// <param name="commandText"></param>
    /// <param name="commandParameters"></param>
    /// <returns></returns>
    public static int ExecuteNonQuery(CommandType commandType, string commandText, params SqlParameter[] commandParameters)
    {
        SqlCommand cmd = new SqlCommand();
        int val = 0;
        using (SqlConnection conn = new SqlConnection(conStr))
        {
            PrepareCommand(cmd, commandType, conn, commandText, commandParameters);
            try
            {
                val = cmd.ExecuteNonQuery();
                cmd.Parameters.Clear();
            }
            catch (Exception)
            {

                throw;
            }
            finally
            {
                conn.Close();
            }
            return val;
        }
    }
    /// <summary>
    /// 使用DataTable作为数据源
    /// </summary>
    /// <param name="commandType"></param>
    /// <param name="commandText"></param>
    /// <param name="commandParameters"></param>
    /// <returns></returns>
    public static DataTable ExecuteDataTable(CommandType commandType, string commandText, params SqlParameter[] commandParameters)
    {
        using (SqlConnection conn = new SqlConnection(conStr))
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, commandType, conn, commandText, commandParameters);
            using (SqlDataAdapter da = new SqlDataAdapter(cmd))
            {
                DataTable dt = new DataTable();
                da.Fill(dt);
                cmd.Parameters.Clear();
                return dt;
            }
        }
    }

    /// <summary>
    /// 通过反射将Table数据转换成对应的实体提盒
    /// </summary>
    /// <typeparam name="T">实体</typeparam>
    /// <param name="dt">数据表</param>
    /// <returns></returns>
    public static List<T> TableToEntity<T>(DataTable dt) where T : class,new()
    {
        Type type = typeof(T);
        List<T> list = new List<T>();
        foreach (DataRow row in dt.Rows)
        {
            PropertyInfo[] pArray = type.GetProperties();
            T entity = new T();
            foreach (PropertyInfo p in pArray)
            {
                if (row[p.Name] is Int64)
                {
                    p.SetValue(entity, Convert.ToInt32(row[p.Name]), null);
                    continue;
                }
                p.SetValue(entity, row[p.Name], null);
            }
            list.Add(entity);
        }
        return list;
    }

    /// <summary>
    /// 使用DataSet作为数据源
    /// </summary>
    /// <param name="commandType"></param>
    /// <param name="commandText"></param>
    /// <param name="commandParameters"></param>
    /// <returns></returns>
    public static DataSet ExecuteDataset(CommandType commandType, string commandText, params SqlParameter[] commandParameters)
    {
        using (SqlConnection conn = new SqlConnection(conStr))
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, commandType, conn, commandText, commandParameters);
            using (SqlDataAdapter da = new SqlDataAdapter(cmd))
            {
                DataSet ds = new DataSet();
                da.Fill(ds);
                cmd.Parameters.Clear();
                return ds;
            }
        }
    }
    /// <summary>
    /// 传递参数
    /// </summary>
    /// <param name="cmd"></param>
    /// <param name="commandType"></param>
    /// <param name="conn"></param>
    /// <param name="commandText"></param>
    /// <param name="cmdParms"></param>
    private static void PrepareCommand(SqlCommand cmd, CommandType commandType, SqlConnection conn, string commandText, SqlParameter[] cmdParms)
    {
        //打开连接
        if (conn.State != ConnectionState.Open)
            conn.Open();

        //设置SqlCommand对象
        cmd.Connection = conn;
        cmd.CommandText = commandText;
        cmd.CommandType = commandType;

        if (cmdParms != null)
        {
            foreach (SqlParameter parm in cmdParms)
                cmd.Parameters.Add(parm);
        }
    }
    #endregion
}